#!/bin/sh

[ -n "$include_indexmeta" ] && return 0
include_indexmeta="$0"

. "$_EXEC/cgilite/storage.sh"
file_pattern='^.*\.(mov|ts|mpg|mpeg|mp4|m4v|avi|mkv|flv|sfv|wmv|ogm|ogv|webm|iso|rmvb)$'

meta_name() {
  local fn
  fn="$1"; fn="${fn##*/}"; fn="${fn%.*}"
  STRING "$fn"; printf '\r'
}

meta_line() {
  local video l w h
  video="$1"

  read l h w <<__EOF
  $(printf '' \
    | mplayer -input nodefault-bindings -nosound -vo null -identify -frames 0 "$video" 2>&- \
    | sort | sed -rn '
      s:ID_LENGTH=(.*)(\..*)$:\1:p;
      s:ID_VIDEO_HEIGHT=(.*):\1:p;
      s:ID_VIDEO_WIDTH=(.*):\1:p;' \
    | tr '\n' ' '
  )
__EOF
  printf '%i\t%i\t%i\ttags=\tcomment=\t%s\n' \
         "${l-0}" "${w-0}" "${h-0}" "$(meta_name "$video")"
}

meta_file(){
  local file meta name
  file="$1"
  meta="${file%/*}/.index/meta"
  name="$(meta_name "$file")"

  if [ -d "${meta%/meta}" ] && LOCK "$meta"; then
    grep -avF "	${name}" "$meta" >"$meta.tmp"
    meta_line "$file" \
    | tee -a "$meta.tmp"
    mv "$meta.tmp" "$meta"
    RELEASE "$meta"
  fi
}

meta_purge(){
  local file meta name
  file="$1"
  meta="${file%/*}/.index/meta"
  name="$(meta_name "$file")"

  if [ -d "${meta%/meta}" ] && LOCK "$meta"; then
    grep -avF "	${name}" "$meta" >"${meta}.tmp"
    grep -aF "	${name}" "$meta" >>"${meta}.trash"
    mv "${meta}.tmp" "$meta"
    RELEASE "$meta"
  fi
}

meta_info(){
  local file meta
  file="$1"; meta="${file%/*}/.index/meta"

  if [ -d "${meta%/meta}" ]; then
    grep -aF "	$(meta_name "$file")" "$meta" \
    | grep -m1 -axE '[0-9]+	[0-9]+	[0-9]+	tags=[^	]*	comment=[^	]*	.+' \
    || meta_file "$file"
  else
    printf '0\t0\t0\ttags=\tcomment=\t%s\r' "$(meta_name "$file")"
  fi
}

meta_dir(){
  local dir meta v
  dir="${1}"
  meta="${dir}/.index/meta"

  if [ -d "$dir/.index" -a \! -f "$meta" ] && LOCK "$meta"; then
    touch "$meta"  # preliminary touch to prevent concurrent generators
    find "$dir" -type f -mindepth 1 -maxdepth 1 \
    | grep -aE "$file_pattern" \
    | while read -r v; do
      meta_line "$v"
    done >"$meta"

    RELEASE "$meta"
  elif [ -d "$dir/.index" -a "$dir" -nt "$meta" ] && LOCK "$meta"; then
    cp -p "$meta" "$meta.ref"; touch "$meta"
    find "$dir" -type f -newer "$meta.ref" \
         -mindepth 1 -maxdepth 1 \
    | grep -aE "$file_pattern" \
    | while read -r v; do
      grep -qF "	$(meta_name "$v")" "$meta" \
      || meta_line "$v"
    done >>"$meta"
    sort -u "$meta" >"$meta.ref"
    mv "$meta.ref" "$meta"

    RELEASE "$meta"
  fi
}
